﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ImpressaoDiretaRV
{
    public partial class FormPrincipal : Form
    {
        public FormPrincipal()
        {
            InitializeComponent();
        }

        private void btExibir_Click(object sender, EventArgs e)
        {
            var ds = CriarDataSet();
            FormRelatorio.ShowReport("Relatorio.rdlc", false, new Dictionary<string, object>() { { "DataSetRelatorio", ds.Item } });
        }

        private DataSetRelatorio CriarDataSet()
        {
            var ds = new DataSetRelatorio();

            for (int c = 1; c <= 200; c++)
            {
                ds.Item.AddItemRow(c, string.Format("Item {0}", c));
            }

            return ds;
        }

        private void btImprimir_Click(object sender, EventArgs e)
        {
            using (var ds = CriarDataSet())
            using (var relatorio = new Microsoft.Reporting.WinForms.LocalReport())
            {
                relatorio.ReportPath = "Relatorio.rdlc";
                relatorio.DataSources.Add(new Microsoft.Reporting.WinForms.ReportDataSource("DataSetRelatorio", (DataTable)ds.Item));

                //Exportar(relatorio);
                //Imprimir(relatorio);

                using (var rpd = new PrintReportSample.ReportPrintDocument(relatorio))
                {
                    rpd.Print();
                }
            }
        }

        private void Exportar(Microsoft.Reporting.WinForms.LocalReport relatorio)
        {
            Microsoft.Reporting.WinForms.Warning[] warnings;
            LimparStreams();
            relatorio.Render("image", CriarDeviceInfo(relatorio), CreateStreamCallback, out warnings);
        }

        private void Imprimir(Microsoft.Reporting.WinForms.LocalReport relatorio)
        {
            using (var pd = new System.Drawing.Printing.PrintDocument())
            {
                pd.PrinterSettings.PrinterName = "PrimoPDF";
                var pageSettings = new System.Drawing.Printing.PageSettings();
                var pageSettingsRelatorio = relatorio.GetDefaultPageSettings();
                pageSettings.PaperSize = pageSettingsRelatorio.PaperSize;
                pageSettings.Margins = pageSettingsRelatorio.Margins;
                pd.DefaultPageSettings = pageSettings;

                pd.PrintPage += Pd_PrintPage;
                _streamAtual = 0;
                pd.Print();
            }
        }

        private int _streamAtual;
        private void Pd_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            var stream = _streams[_streamAtual];
            stream.Seek(0, System.IO.SeekOrigin.Begin);

            using (var metadata = new System.Drawing.Imaging.Metafile(stream))
            {
                e.Graphics.DrawImage(metadata, e.PageBounds);
            }

            _streamAtual++;
            e.HasMorePages = _streamAtual < _streams.Count;
        }

        private List<System.IO.Stream> _streams = new List<System.IO.Stream>();
        public System.IO.Stream CreateStreamCallback(string name, string extension, Encoding encoding, string mimeType, bool willSeek)
        {
            var stream = new System.IO.MemoryStream();
            _streams.Add(stream);
            return stream;
        }

        private string CriarDeviceInfo(Microsoft.Reporting.WinForms.LocalReport relatorio)
        {
            var pageSettings = relatorio.GetDefaultPageSettings();
            return string.Format(
                System.Globalization.CultureInfo.InvariantCulture,
                @"<DeviceInfo>
                    <OutputFormat>EMF</OutputFormat>
                    <PageWidth>{0}in</PageWidth>
                    <PageHeight>{1}in</PageHeight>
                    <MarginTop>{2}in</MarginTop>
                    <MarginLeft>{3}in</MarginLeft>
                    <MarginRight>{4}in</MarginRight>
                    <MarginBottom>{5}in</MarginBottom>
                </DeviceInfo>",
                pageSettings.PaperSize.Width / 100m, pageSettings.PaperSize.Height / 100m, pageSettings.Margins.Top / 100m, pageSettings.Margins.Left / 100m, pageSettings.Margins.Right / 100m, pageSettings.Margins.Bottom / 100m);
        }

        private void LimparStreams()
        {
            foreach (var stream in _streams)
            {
                stream.Dispose();
            }
            _streams.Clear();
        }
    }
}
