﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ImpressaoDiretaRV
{
    public partial class FormRelatorio : Form
    {
        private FormRelatorio()
        {
            InitializeComponent();
        }

        private FormRelatorio(string path, bool isEmbeddedResource, Dictionary<string, object> dataSources, Dictionary<string, object> reportParameters = null)
        {
            InitializeComponent();

            // path + isEmbeddedResource.
            if (isEmbeddedResource)
                this.reportViewer.LocalReport.ReportEmbeddedResource = path;
            else
                this.reportViewer.LocalReport.ReportPath = path;

            // dataSources.
            foreach (var dataSource in dataSources)
            {
                var reportDataSource = new Microsoft.Reporting.WinForms.ReportDataSource(dataSource.Key, dataSource.Value);
                this.reportViewer.LocalReport.DataSources.Add(reportDataSource);
            }

            // reportParameters.
            if (reportParameters != null)
            {
                var reportParameterCollection = new List<Microsoft.Reporting.WinForms.ReportParameter>();

                foreach (var parameter in reportParameters)
                {
                    var reportParameter = new Microsoft.Reporting.WinForms.ReportParameter(parameter.Key, parameter.Value.ToString());
                    reportParameterCollection.Add(reportParameter);
                }

                this.reportViewer.LocalReport.SetParameters(reportParameterCollection);
            }
        }

        public static void ShowReport(string path, bool isEmbeddedResource, Dictionary<string, object> dataSources, Dictionary<string, object> reportParameters = null)
        {
            var formRelatorio = new FormRelatorio(path, isEmbeddedResource, dataSources, reportParameters);
            formRelatorio.Show();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.reportViewer.RefreshReport();
        }
    }
}
